# puppeteer-evals

`puppeteer-evals` is a Node.js package that provides a set of utility functions for working with Puppeteer, making it easier to extract information from web pages.

## Installation

You can install `puppeteer-evals` using npm:

```bash
npm install puppeteer-evals
```

## Usage

First, import and initialize the package:

```javascript
const evals = require('puppeteer-evals');
evals();

// Or, you can use it in one line:
// require('puppeteer-evals')();
```

After initialization, all functions can be called directly without referencing 'evals' again.

Example usage (assuming you've set up Puppeteer and have a 'page' object):

```javascript
const browser = await puppeteer.launch();
const page = await browser.newPage();
await page.goto('https://example.com');

// Now you can use the evals functions:
const inputValues = await getInputs(page);
const buttonTexts = await getButtons(page);
const usernameValue = await getAttribute(page, 'input[name="username"]', 'value');

await browser.close();
```

## Available Methods

Here's a list of available methods:

- `getInputs(page)`: Get all input values
- `getTextareas(page)`: Get all textarea values
- `getSelects(page)`: Get all select values
- `getCheckboxes(page)`: Get all checkbox values
- `getRadios(page)`: Get all radio values
- `getButtons(page)`: Get all button texts
- `getLinks(page)`: Get all link hrefs
- `getInteractables(page)`: Get all interactable elements
- `getPageText(page)`: Get the page text
- `getPageHtml(page)`: Get the page HTML
- `getPageAttributes(page, selector, attribute)`: Get attributes of elements matching a selector
- `getPageRefs(page)`: Get all page references (links and sources)
- `findElement(page, selector)`: Find an element
- `findAllElements(page, selector)`: Find all elements matching a selector
- `getInnerText(page, selector)`: Get the inner text of an element
- `getInnerHTML(page, selector)`: Get the inner HTML of an element
- `getAttribute(page, selector, attribute)`: Get an attribute of an element

For more details on each method, you can use the `help()` function after initialization.

## Contributing

Contributions are welcome! Please feel free to submit a Pull Request.

## License

This project is licensed under the MIT License.
