
class Evals {

	static getInputs (page) {
    return page.evaluate(() => {
      const inputs = Array.from(document.querySelectorAll('input'));
      return inputs.map(input => input.value);
    });
  };

	static getTextareas (page) {
    return page.evaluate(() => {
      const textareas = Array.from(document.querySelectorAll('textarea'));
      return textareas.map(textarea => textarea.value);
    });
  };

	static getSelects (page) {
    return page.evaluate(() => {
      const selects = Array.from(document.querySelectorAll('select'));
      return selects.map(select => select.value);
    });
  };

	static getCheckboxes (page) {
    return page.evaluate(() => {
      const checkboxes = Array.from(document.querySelectorAll('input[type="checkbox"]'));
      return checkboxes.map(checkbox => checkbox.checked);
    });
  };

	static getRadios (page) {
    return page.evaluate(() => {
      const radios = Array.from(document.querySelectorAll('input[type="radio"]'));
      return radios.map(radio => radio.checked);
    });
  };

	static getButtons (page) {
    return page.evaluate(() => {
      const buttons = Array.from(document.querySelectorAll('button'));
      return buttons.map(button => button.innerText);
    });
  };

	static getLinks (page) {
    return page.evaluate(() => {
      const links = Array.from(document.querySelectorAll('a'));
      return links.map(link => link.href);
    });
  };

	static getInteractables (page) {
    return page.evaluate(() => {
      const interactables = {};
      interactables['buttons'] = Array.from(document.querySelectorAll('button'));
      interactables['links'] = Array.from(document.querySelectorAll('a'));
      interactables['inputs'] = Array.from(document.querySelectorAll('input'));
      interactables['textareas'] = Array.from(document.querySelectorAll('textarea'));
      interactables['selects'] = Array.from(document.querySelectorAll('select'));
      interactables['checkboxes'] = Array.from(document.querySelectorAll('input[type="checkbox"]'));
      interactables['radios'] = Array.from(document.querySelectorAll('input[type="radio"]'));
      return interactables;
    });
  };

  static getPageText (page) {
    return page.evaluate(() => {
      return document.body.innerText;
    });
  };

  static getPageHtml (page) {
    return page.evaluate(() => {
      return document.body.innerHTML;
    });
  };

  static getPageAttributes (page, selector, attribute) {
    return page.evaluate((selector, attribute) => {
      const elements = Array.from(document.querySelectorAll(selector));
      return elements.map(element => element.getAttribute(attribute));
    }, selector, attribute);
  };

  static getPageRefs (page) {
    return page.evaluate(() => {
      const unique = (arr) => arr.filter((i, index) => arr.indexOf(i) === index);
      unique([...document.body.querySelectorAll('*')].map(x => {
        let href = x.getAttribute('href');
        let src = x.getAttribute('src');
        let link = href && href.length > 0 ? href : src;
        if (link) {
          if (link.startsWith('http')) return link;
          if (link.startsWith('/')) return window.location.origin + link;
          return window.location.origin + window.location.pathname.substring(0, window.location.pathname.lastIndexOf('/')) + '/' + link;
        }
      }).filter(x => x?.length && !x.includes('#')));
    });
  };

  static findElement (page, selector) {
    return page.evaluate((selector) => {
      return document.querySelector(selector);
    }, selector);
  };

  static findAllElements (page, selector) {
    return page.evaluate((selector) => {
      return [...document.querySelectorAll(selector)];
    }, selector);
  };

  static getInnerText (page, selector) {
    return page.evaluate((selector) => {
      return document.querySelector(selector).innerText;
    }, selector);
  };

  static getInnerHTML (page, selector) {
    return page.evaluate((selector) => {
      return document.querySelector(selector).innerHTML;
    }, selector);
  };

  static getAttribute (page, selector, attribute) {
    return page.evaluate((selector, attribute) => {
      return document.querySelector(selector).getAttribute(attribute);
    }, selector, attribute);
  };

  static listMethods () {
    return {
      'getInputs': { description: 'Get all input values', takes: ['page'], returns: 'Array' },
      'getTextareas': { description: 'Get all textarea values', takes: ['page'], returns: 'Array' },
      'getSelects': { description: 'Get all select values', takes: ['page'], returns: 'Array' },
      'getCheckboxes': { description: 'Get all checkbox values', takes: ['page'], returns: 'Array' },
      'getRadios': { description: 'Get all radio values', takes: ['page'], returns: 'Array' },
      'getButtons': { description: 'Get all button texts', takes: ['page'], returns: 'Array' },
      'getLinks': { description: 'Get all link hrefs', takes: ['page'], returns: 'Array' },
      'getInteractables': { description: 'Get all interactables', takes: ['page'], returns: 'Object' },
      'getPageText': { description: 'Get the page text', takes: ['page'], returns: 'String' },
      'getPageHtml': { description: 'Get the page html', takes: ['page'], returns: 'String' },
      'getPageAttributes': { description: 'Get the page attributes', takes: ['page', 'selector', 'attribute'], returns: 'Array' },
      'getPageRefs': { description: 'Get the page references', takes: ['page'], returns: 'Array' },
      'findElement': { description: 'Find an element', takes: ['page', 'selector'], returns: 'Element' },
      'findAllElements': { description: 'Find all elements', takes: ['page', 'selector'], returns: 'Array' },
      'getInnerText': { description: 'Get the inner text', takes: ['page', 'selector'], returns: 'String' },
      'getInnerHTML': { description: 'Get the inner html', takes: ['page', 'selector'], returns: 'String' },
      'getAttribute': { description: 'Get the attribute', takes: ['page', 'selector', 'attribute'], returns: 'String' },
    }
  };

  static help () {
    console.log(Evals.listMethods());
  }

  static getMethods () {
    return Object.getOwnPropertyNames(Evals).filter(x => typeof Evals[x] === 'function');
  }

}

module.exports = Evals;

if (require.main === module) {
  console.log(Evals.listMethods());
}

